#  Esercizio Trasferte

# DATI
set N;                  # Insieme delle squadre
param n := card(N);     # Numero di squadre
set T := 1..2*(n-1);    # Insieme indicizzato dei turni di gara
param d {N,N};          # Matrice delle distanze tra le citt

# VARIABILI
var x {N,N,T} binary;   # Assegnamento: incontro i-j nel turno t
var y1 {N,N} binary;    # y1[i,j] = 1 se e solo se la squadra j si sposta con andata e ritorno nella citt della squadra i
var y2 {N,N,N} binary;  # y2[i,j,k] = 1 se e solo se la squadra k gioca in trasferta consecutivamente contro la squadra i e contro la squadra j
var delta {N};          # Distanza totale percorsa da una squadra
var Delta;              # Var. ausiliaria per la linearizzazione dell'obiettivo 2

# VINCOLI
# In ogni turno t, ogni squadra j gioca una partita
s.t. UnaPartita {j in N, t in T}: 
  sum {i in N} (x[i,j,t] + x[j,i,t]) = 1;

# Nessuna squadra pu mai giocare con s stessa
s.t. NoLoop {j in N, t in T}: 
  x[j,j,t] = 0;

# I due gironi di andata e ritorno devono essere simmetrici
s.t. Simmetrici {i in N, j in N, t in T: (i<>j) and (t <= n-1)}:
  x[i,j,t] = x[j,i,t+(n-1)];

# Ogni coppia di squadre si deve incontrare una volta in ogni girone
s.t. Coppia {i in N, j in N: i <> j}:
  sum {t in T: t <= n-1} (x[i,j,t] + x[j,i,t]) = 1;

# In ogni girone, ogni squadra gioca un numero di partite in casa ed un numero di partite in trasferta che differiscono al massimo di 1.
s.t. Diff1_1 {j in N}:
  sum {t in T: t <= (n-1)} sum {i in N} (x[i,j,t] - x[j,i,t]) <=  1;
s.t. Diff1_2 {j in N}:
  sum {t in T: t <= (n-1)} sum {i in N} (x[i,j,t] - x[j,i,t]) >= -1;

# Nessuna squadra deve giocare tre incontri consecutivi in casa o in trasferta
s.t. No3home {j in N, t in T: (t>1) and (t<card(T))}:
  sum {i in N} (x[j,i,t-1] + x[j,i,t] + x[j,i,t+1]) <= 2;
s.t. No3out {j in N, t in T: (1<t) and (t<card(T))}:
  sum {i in N} (x[i,j,t-1] + x[i,j,t] + x[i,j,t+1]) <= 2;

# La squadra j percorre un viaggio di andata e ritorno nella citt i in caso di trasferta singola
s.t. TrasfSingola_t {i in N, j in N, t in T: (i <> j) and (1 < t) and (t < card(T))}:
  sum {k in N} x[j,k,t-1] + x[i,j,t] + sum {k in N} x[j,k,t+1] - 2 <= y1[i,j];
s.t. TrasfSingola_1 {i in N, j in N: (i <> j)}:
  x[i,j,1] + sum {k in N} x[j,k,2] - 1 <= y1[i,j];
s.t. TrasfSingola_T {i in N, j in N: (i <> j)}:
  sum {k in N} x[j,k,card(T)-1] + x[i,j,card(T)] - 1 <= y1[i,j];

# La squadra k percorre un viaggio di due tappe nelle citt i e j in caso di trasferte consecutive
s.t. TrasfDoppia {i in N, j in N, k in N, t in T: (i<>j) and (i<>k) and (j<>k) and (t<card(T))}:
  x[i,k,t] + x[j,k,t+1] - 1 <= y2[i,j,k];

# OBIETTIVI

# Distanza totale percorsa da ogni squadra
s.t. Def_delta {k in N}:
  delta[k] = (sum {i in N} 2 * d[i,k] * y1[i,k] + 
              sum {i in N, j in N: (i<>j) and (i<>k) and (j<>k)} (d[k,i] + d[i,j] + d[j,k]) * y2[i,j,k]
             );

# Obiettivo 1: minimizzare la distanza totale percorsa da tutte le squadre
minimize z1: sum {k in N} delta[k];

# Obiettivo 2: minimizzare la massima differenza tra le distanze percorse dalle squadre
#minimize z2: Delta;
s.t. MinMaxDiff {k1 in N, k2 in N: k1<>k2}: 
  Delta >= delta[k1] - delta[k2];

##############################
data;

set N := A B C D E F G H I J K L;

param d:  
   A   B   C   D   E   F   G   H   I   J   K   L :=
A   0 116 132 156 169 174 187 190 200 215 221 229
B 116   0 129 141 158 169 185 189 198 210 214 225
C 132 129   0 128 142 160 173 182 195 204 208 222
D 156 141 128   0 114 138 150 180 190 197 201 207
E 169 158 142 114   0 120 145 171 178 179 185 194
F 174 169 160 138 120   0 126 146 157 171 177 184
G 187 185 173 150 145 126   0 130 140 161 165 191
H 190 189 182 180 171 146 130   0 125 141 155 174
I 200 198 195 190 178 157 140 125   0 113 135 172
J 215 210 204 197 179 171 161 141 113   0 131 120
K 221 214 208 201 185 177 165 155 135 131   0 118
L 229 225 222 207 194 184 191 174 172 120 118   0;

end;
